@@ -82,6 +82,14 @@
 #include <syslog.h>
 #include <assert.h>
 
+#ifdef __linux__
+#include <fcntl.h>
+#include <sys/ioctl.h>
+
+/* from <linux/cdrom.h> */
+#define CDROM_GET_CAPABILITY	0x5331	/* get capabilities */
+#endif /* __linux__ */
+
 #include "autopartkit.h"
 
 /*
@@ -93,7 +101,7 @@
 
 #define FAT_MINSIZE_FACTOR 1.33
 
-#define MAX_PARTITIONS 10
+#define MAX_PARTITIONS 30
 
 #define PART_SIZE_BYTE(device, part) ((part)->geom.length * (device)->sector_size)
 
@@ -305,6 +313,25 @@ static int is_root(const char *mountpoint)
     return (0 == strcmp(mountpoint, "/") );
 }
 
+#ifdef __linux__
+static int is_cdrom(const char *path)
+{
+    int fd;
+    int ret;
+
+    fd = open(path, O_RDONLY | O_NONBLOCK);
+    ret = ioctl(fd, CDROM_GET_CAPABILITY, NULL);
+    close(fd);
+
+    if (ret >= 0)
+	return 1;
+    else
+	return 0;
+}
+#else /* !__linux__ */
+#define is_cdrom(path) 0
+#endif /* __linux__ */
+
 /*
  * Step 1 & 2: Discover and select the device
  *
@@ -326,6 +353,7 @@ static PedDevice* choose_device(void)
 #if defined(fordebian)
     char       device_list[LIST_SIZE], default_device[128], table[TABLE_SIZE];
     char      *ptr_list, *ptr_table;
+    int        num_devices = 0;
     DeviceStats *stats;
 #endif /* fordebian */
 
@@ -334,6 +362,10 @@ static PedDevice* choose_device(void)
     disable_kmsg(0);
 
     dev = ped_device_get_next(NULL);
+    while (dev != NULL && (dev->read_only || is_cdrom(dev->path)))
+    {
+	dev = ped_device_get_next(dev);
+    }
 
     if (dev == NULL)
     {
@@ -352,13 +384,6 @@ static PedDevice* choose_device(void)
 	return dev;
     }
 #if defined(fordebian)
-    if (dev->next == NULL)
-    {
-	/* There's only a single device detected */
-	return dev;
-    }
-
-
     default_device[0] = '\0';
     device_list[0] = '\0';
     ptr_list = device_list;
@@ -369,6 +394,9 @@ static PedDevice* choose_device(void)
 	if (strstr(dev->path, "dev/ide/") || strstr(dev->path, "dev/scsi/"))
 	    continue;
 #endif
+	if (dev->read_only || is_cdrom(dev->path))
+	    continue;
+	++num_devices;
 	stats = get_device_stats(dev);
 	/*
 	 * This is the right way to do unfortunately I want insecable
@@ -413,6 +441,12 @@ static PedDevice* choose_device(void)
 	free(stats);
     }
 
+    if (num_devices == 1)
+    {
+	/* There's only a single usable device detected */
+	return ped_device_get(default_device);
+    }
+
     dev = NULL;
     nb_try = 0;
     while (dev == NULL)
@@ -905,7 +939,7 @@ makefs(const char *devpath, const char *fstype)
     else if (0 == strcmp("ext2", fstype))
         mkfs = "/sbin/mkfs.ext2";
     else if (0 == strcmp("ext3", fstype))
-        mkfs = "/sbin/mkfs.ext3";
+        mkfs = "/sbin/mkfs.ext3 -O resize_inode";
     else if (0 == strcmp("jfs", fstype))
         mkfs = "/sbin/mkfs.jfs";
     else if (0 == strcmp("xfs", fstype))
@@ -1305,9 +1339,9 @@ make_partitions(const diskspace_req_t *space_reqs, PedDevice *devlist)
 				lvm_reqs[lvm_reqs_index].maxsize);
                 lvm_reqs_index++;
             }
-            lvm_reqs[lvm_reqs_index].mountpoint = NULL;
             free(identmatch);
         }
+	lvm_reqs[lvm_reqs_index].mountpoint = NULL;
 
         /* Reduce vg size by LVM_SPEND_FACTOR... */
 	autopartkit_log(1, "Vg %s - %d to use\n",
